@@ -55,41 +55,41 @@ litl_write_trace_t* litl_write_init_trace(const uint32_t buf_size);
 
 /**
  * \ingroup litl_write_init
- * \brief Activates buffer flush
+ * \brief Enable buffer flush. By default, it is disabled
  * \param trace A pointer to the event recording object
  */
 void litl_write_buffer_flush_on(litl_write_trace_t* trace);
 
 /**
  * \ingroup litl_write_init
- * \brief Deactivates buffer flush. By default, it is activated
+ * \brief Disable buffer flush.
  * \param trace A pointer to the event recording object
  */
 void litl_write_buffer_flush_off(litl_write_trace_t* trace);
 
 /**
  * \ingroup litl_write_init
- * \brief Activates thread safety
+ * \brief Enable thread safety
  * \param trace A pointer to the event recording object
  */
 void litl_write_thread_safety_on(litl_write_trace_t* trace);
 
 /**
  * \ingroup litl_write_init
- * \brief Deactivates thread safety. By default, it is activated
+ * \brief Disable thread safety. By default, it is enabled
  * \param trace A pointer to the event recording object
  */
 void litl_write_thread_safety_off(litl_write_trace_t* trace);
 
 /**
  * \ingroup litl_write_init
- * \brief Activates recording tid
+ * \brief Enable recording tid
  */
 void litl_write_tid_recording_on(litl_write_trace_t* trace);
 
 /**
  * \ingroup litl_write_init
- * \brief Deactivates recording tid. By default, it is activated
+ * \brief Disable recording tid. By default, it is enabled
  * \param trace A pointer to the event recording object
  */
 void litl_write_tid_recording_off(litl_write_trace_t* trace);
@@ -123,8 +123,9 @@ void litl_write_set_filename(litl_write_trace_t* trace, char* filename);
  * \brief Records a regular event without parameters
  * \param trace A pointer to the event recording object
  * \param code An event code
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_0(litl_write_trace_t* trace, litl_code_t code);
+litl_t* litl_write_probe_reg_0(litl_write_trace_t* trace, litl_code_t code);
 
 /**
  * \ingroup litl_write_reg
@@ -132,9 +133,10 @@ void litl_write_probe_reg_0(litl_write_trace_t* trace, litl_code_t code);
  * \param trace A pointer to the event recording object
  * \param code An event code
  * \param param1 1st parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_1(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1);
+litl_t* litl_write_probe_reg_1(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1);
 
 /**
  * \ingroup litl_write_reg
@@ -143,9 +145,10 @@ void litl_write_probe_reg_1(litl_write_trace_t* trace, litl_code_t code,
  * \param code An event code
  * \param param1 1st parameter for this event
  * \param param2 2nd parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_2(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2);
+litl_t* litl_write_probe_reg_2(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2);
 
 /**
  * \ingroup litl_write_reg
@@ -155,10 +158,11 @@ void litl_write_probe_reg_2(litl_write_trace_t* trace, litl_code_t code,
  * \param param1 1st parameter for this event
  * \param param2 2nd parameter for this event
  * \param param3 3rd parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_3(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3);
+litl_t* litl_write_probe_reg_3(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3);
 
 /**
  * \ingroup litl_write_reg
@@ -169,10 +173,11 @@ void litl_write_probe_reg_3(litl_write_trace_t* trace, litl_code_t code,
  * \param param2 2nd parameter for this event
  * \param param3 3rd parameter for this event
  * \param param4 4th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_4(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4);
+litl_t* litl_write_probe_reg_4(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4);
 
 /**
  * \ingroup litl_write_reg
@@ -184,11 +189,12 @@ void litl_write_probe_reg_4(litl_write_trace_t* trace, litl_code_t code,
  * \param param3 3rd parameter for this event
  * \param param4 4th parameter for this event
  * \param param5 5th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_5(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5);
+litl_t* litl_write_probe_reg_5(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4,
+			       litl_param_t param5);
 
 /**
  * \ingroup litl_write_reg
@@ -201,11 +207,12 @@ void litl_write_probe_reg_5(litl_write_trace_t* trace, litl_code_t code,
  * \param param4 4th parameter for this event
  * \param param5 5th parameter for this event
  * \param param6 6th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_6(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6);
+litl_t* litl_write_probe_reg_6(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4,
+			       litl_param_t param5, litl_param_t param6);
 
 /**
  * \ingroup litl_write_reg
@@ -219,12 +226,13 @@ void litl_write_probe_reg_6(litl_write_trace_t* trace, litl_code_t code,
  * \param param5 5th parameter for this event
  * \param param6 6th parameter for this event
  * \param param7 7th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_7(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7);
+litl_t* litl_write_probe_reg_7(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4,
+			       litl_param_t param5, litl_param_t param6,
+			       litl_param_t param7);
 
 /**
  * \ingroup litl_write_reg
@@ -239,12 +247,13 @@ void litl_write_probe_reg_7(litl_write_trace_t* trace, litl_code_t code,
  * \param param6 6th parameter for this event
  * \param param7 7th parameter for this event
  * \param param8 8th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_8(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7, litl_param_t param8);
+litl_t* litl_write_probe_reg_8(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4,
+			       litl_param_t param5, litl_param_t param6,
+			       litl_param_t param7, litl_param_t param8);
 
 /**
  * \ingroup litl_write_reg
@@ -260,13 +269,14 @@ void litl_write_probe_reg_8(litl_write_trace_t* trace, litl_code_t code,
  * \param param7 7th parameter for this event
  * \param param8 8th parameter for this event
  * \param param9 9th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_9(litl_write_trace_t* trace, litl_code_t code,
-                            litl_param_t param1, litl_param_t param2,
-                            litl_param_t param3, litl_param_t param4,
-                            litl_param_t param5, litl_param_t param6,
-                            litl_param_t param7, litl_param_t param8,
-                            litl_param_t param9);
+litl_t* litl_write_probe_reg_9(litl_write_trace_t* trace, litl_code_t code,
+			       litl_param_t param1, litl_param_t param2,
+			       litl_param_t param3, litl_param_t param4,
+			       litl_param_t param5, litl_param_t param6,
+			       litl_param_t param7, litl_param_t param8,
+			       litl_param_t param9);
 
 /**
  * \ingroup litl_write_reg
@@ -283,13 +293,14 @@ void litl_write_probe_reg_9(litl_write_trace_t* trace, litl_code_t code,
  * \param param8 8th parameter for this event
  * \param param9 9th parameter for this event
  * \param param10 10th parameter for this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_reg_10(litl_write_trace_t* trace, litl_code_t code,
-                             litl_param_t param1, litl_param_t param2,
-                             litl_param_t param3, litl_param_t param4,
-                             litl_param_t param5, litl_param_t param6,
-                             litl_param_t param7, litl_param_t param8,
-                             litl_param_t param9, litl_param_t param10);
+litl_t* litl_write_probe_reg_10(litl_write_trace_t* trace, litl_code_t code,
+				litl_param_t param1, litl_param_t param2,
+				litl_param_t param3, litl_param_t param4,
+				litl_param_t param5, litl_param_t param6,
+				litl_param_t param7, litl_param_t param8,
+				litl_param_t param9, litl_param_t param10);
 
 /*** Raw events ***/
 
@@ -300,9 +311,10 @@ void litl_write_probe_reg_10(litl_write_trace_t* trace, litl_code_t code,
  * \param code An event code
  * \param size Size (in Bytes) of the data to store
  * \param data Data to store with this event
+ * \return a pointer to the event that was recorded or NULL in case of error
  */
-void litl_write_probe_raw(litl_write_trace_t* trace, litl_code_t code,
-                          litl_size_t size, litl_data_t data[]);
+litl_t* litl_write_probe_raw(litl_write_trace_t* trace, litl_code_t code,
+			     litl_size_t size, litl_data_t data[]);
 
 /*** Internal-use macros ***/
 
@@ -339,7 +351,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
  * \param code An event code
  */
 #define litl_write_probe_pack_0(trace,					\
-				code) do {				\
+				code,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE;					\
     litl_t* p_evt = __litl_write_get_event(trace,			\
 					   LITL_TYPE_PACKED,		\
@@ -347,6 +360,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
     if(p_evt){								\
       p_evt->parameters.packed.size = total_size - LITL_BASE_SIZE;	\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -358,7 +372,9 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
  */
 #define litl_write_probe_pack_1(trace,					\
 				code,					\
-				param1) do {				\
+				param1,					\
+				retval)					\
+  do {									\
     int total_size = LITL_BASE_SIZE + sizeof(param1);			\
     litl_t* p_evt = __litl_write_get_event(trace,			\
 					   LITL_TYPE_PACKED,		\
@@ -369,6 +385,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       void* _ptr_ = &p_evt->parameters.packed.param[0];			\
       __LITL_WRITE_ADD_ARG(_ptr_, param1);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -382,7 +399,9 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 #define litl_write_probe_pack_2(trace,					\
 				code,					\
 				param1,					\
-				param2) do {				\
+				param2,					\
+				retval)					\
+  do {									\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2);	\
     litl_t* p_evt = __litl_write_get_event(trace,			\
 					   LITL_TYPE_PACKED,		\
@@ -394,6 +413,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param1);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param2);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -409,7 +429,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				code,					\
 				param1,					\
 				param2,					\
-				param3) do {				\
+				param3,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) +	\
       sizeof(param3);							\
     litl_t* p_evt = __litl_write_get_event(trace,			\
@@ -423,6 +444,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param2);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param3);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -440,7 +462,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param1,					\
 				param2,					\
 				param3,					\
-				param4) do {				\
+				param4,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4);					\
     litl_t* p_evt = __litl_write_get_event(trace,			\
@@ -455,6 +478,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param3);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param4);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -474,7 +498,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param2,					\
 				param3,					\
 				param4,					\
-				param5) do {				\
+				param5,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4) +sizeof(param5);			\
     litl_t* p_evt = __litl_write_get_event(trace,			\
@@ -490,6 +515,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param4);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param5);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -511,7 +537,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param3,					\
 				param4,					\
 				param5,					\
-				param6) do {				\
+				param6,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4) + sizeof(param5) + sizeof(param6); \
     litl_t* p_evt = __litl_write_get_event(trace,			\
@@ -528,6 +555,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param5);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param6);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -551,7 +579,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param4,					\
 				param5,					\
 				param6,					\
-				param7) do {				\
+				param7,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4) + sizeof(param5) + sizeof(param6) \
       + sizeof(param7);							\
@@ -570,6 +599,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param6);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param7);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -595,7 +625,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param5,					\
 				param6,					\
 				param7,					\
-				param8) do {				\
+				param8,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4) + sizeof(param5) + sizeof(param6) \
       + sizeof(param7) + sizeof(param8);				\
@@ -615,6 +646,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param7);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param8);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -642,7 +674,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				param6,					\
 				param7,					\
 				param8,					\
-				param9) do {				\
+				param9,					\
+				retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4)  + sizeof(param5) + sizeof(param6) \
       + sizeof(param7) + sizeof(param8) + sizeof(param9);		\
@@ -663,6 +696,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param8);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param9);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
@@ -692,7 +726,8 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
 				 param7,				\
 				 param8,				\
 				 param9,				\
-				 param10) do {				\
+				 param10,				\
+				 retval) do {				\
     int total_size = LITL_BASE_SIZE + sizeof(param1) + sizeof(param2) + \
       sizeof(param3) + sizeof(param4) + sizeof(param5) + sizeof(param6) + \
       sizeof(param7) + sizeof(param8) + sizeof(param9) + sizeof(param10); \
@@ -714,6 +749,7 @@ litl_t* __litl_write_get_event(litl_write_trace_t* trace, litl_type_t type,
       __LITL_WRITE_ADD_ARG(_ptr_, param9);				\
       __LITL_WRITE_ADD_ARG(_ptr_, param10);				\
     }									\
+    retval = p_evt;							\
   } while(0)
 
 /**
